/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
    Copyright (C) 2011-2016 OpenFOAM Foundation
    Copyright (C) 2020 OpenCFD Ltd.
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "PJFvPatchField.H"
#include "surfaceFields.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::PJFvPatchField::PJFvPatchField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedJumpFvPatchField<scalar>(p, iF),
    pvj_dictName("PVJDict"),
    pvj_point(vector(0.0,0.0,0.0)),
    pvj_t1d(vector(0.0,0.0,0.0))
{    Info << "PJ-A" << endl; }


Foam::PJFvPatchField::PJFvPatchField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    fixedJumpFvPatchField<scalar>(p, iF),
    pvj_dictName(dict.getOrDefault<word>("PVJDict", "PVJDict")),
    pvj_point(dict.get<vector>("point")),
    pvj_t1d(dict.get<vector>("t1d"))
{

	const fvMesh& mesh=p.boundaryMesh().mesh();
	IOdictionary PVJDict
	    (
	    		IOobject
				(
				pvj_dictName,
				mesh.time().system(),
				mesh,
				IOobject::READ_IF_PRESENT,
				IOobject::NO_WRITE
				)
    );
    #include "readPVJ.H"
    fvPatchField<scalar>::operator=(0.0*p.magSf());
    Info << "PJ-B" << endl;
}


Foam::PJFvPatchField::PJFvPatchField
(
    const PJFvPatchField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedJumpFvPatchField<scalar>(ptf, p, iF, mapper),
    pvj_dictName(ptf.pvj_dictName),
    pvj_point(ptf.pvj_point),
    pvj_t1d(ptf.pvj_t1d)
{    Info << "PJ-C" << endl;}


Foam::PJFvPatchField::PJFvPatchField
(
    const PJFvPatchField& ptf
)
:
    cyclicLduInterfaceField(),
    fixedJumpFvPatchField<scalar>(ptf),
    pvj_dictName(ptf.pvj_dictName),
    pvj_point(ptf.pvj_point),
    pvj_t1d(ptf.pvj_t1d)
{    Info << "PJ-D" << endl;}


Foam::PJFvPatchField::PJFvPatchField
(
    const PJFvPatchField& ptf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedJumpFvPatchField<scalar>(ptf, iF),
    pvj_dictName(ptf.pvj_dictName),
    pvj_point(ptf.pvj_point),
    pvj_t1d(ptf.pvj_t1d)
{    Info << "PJ-E" << endl;}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::PJFvPatchField::updateCoeffs()
{
/*     if (updated())
    {
        return;
    }

*/
    vectorField aeroCoeff = pvj_bn;
    #include "calculatePVJinitializeForces.H"
    forAll(allN, iN)
    {
       #include "calculatePVJforces.H"
	   // Setting jump
       aeroForce[iN] = -q*ca*jumpSign[iN];
    }

    setJump(aeroForce);
    if (internalField().dimensions() == dimPressure)
    {
        setJump
        (
            jump()*patch().lookupPatchField<volScalarField, scalar>("rho")
        );
    }

    if (debug)
    {
        scalar avePressureJump = gAverage(jump());

        Info<< patch().boundaryMesh().mesh().name() << ':'
            << patch().name() << ':'
            << " Average pressure drop :" << avePressureJump
            << endl;
    }

    fixedJumpFvPatchField<scalar>::updateCoeffs();
}


void Foam::PJFvPatchField::write(Ostream& os) const
{
    fixedJumpFvPatchField<scalar>::write(os);
    os.writeEntryIfDifferent<word>("PVJDict", "PVJDict", pvj_dictName);
    os.writeEntry("point", pvj_point);
    os.writeEntry("t1d", pvj_t1d);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchScalarField,
        PJFvPatchField
    );
}

// ************************************************************************* //
