""" PVJ lib """
import numpy

ofDictHeader = """/*--------------------------------*- C++ -*----------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.com
    \\  /    A nd           | Version:  21.12
     \\/     M anipulation  |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     21.12;
    format      ascii;
    class       dictionary;
    object      PVJDict;
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n"""

def fmt(val):
    """ Format  s number for output"""
    return "{:.6f}".format(val)

def getData(fileName,U,area):
    """ Getting data from a text file ordered as [angle, fn, ft1] """
    # Importing data
    data = numpy.loadtxt(fileName)
    # Normalizing data and plotting
    qData = 1.0/2.0*U**2
    data[:,1] = data[:,1]/(qData*area)
    data[:,2] = data[:,2]/(qData*area)
    return data

def getAeroCoeffs(alphas,coeffs):
    """ Gets the aerodynamic coefficeints from Fourier expansion """
    nC = coeffs.shape[0]
    ca = numpy.zeros([alphas.shape[0]])
    for iC in range(nC):
        if coeffs[iC,0]==0.0:
            ca = ca + coeffs[iC,2]*numpy.cos(alphas*numpy.pi/180.0*coeffs[iC,1])
        else:
            ca = ca + coeffs[iC,2]*numpy.sin(alphas*numpy.pi/180.0*coeffs[iC,1])
    return ca        

def getFourierCoeffs(nO,data):
    """ Gets Fourier coefficients for the present barrier """
    alphaRads = data[:,0]*numpy.pi/180.0
    M = list()
    bb = list()
    M.append(numpy.ones(alphaRads.shape[0]))
    bb.append([0.0,0.0,0.0])
    for iO in range(nO):
        M.append(numpy.cos(alphaRads*(iO+1)))
        bb.append([0.0,iO+1,0.0])
        M.append(numpy.sin(alphaRads*(iO+1)))
        bb.append([1.0,iO+1,0.0])
    M = numpy.array(M).T
    bb = numpy.array(bb)
    res = numpy.linalg.lstsq(M,data[:,1],rcond=None)[0]
    bb[:,2] = res
    return bb
    
def getFullyDeflective(theta):
    """ Gets the case of a fully deflective barrier """
    tanTheta = numpy.tan(theta*numpy.pi/180)
    cn = numpy.array([[0.0, 1.0, 2*tanTheta**2],[1.0, 1.0, -2*tanTheta]])
    ct = numpy.array([[0.0, 1.0, -2*tanTheta],[1.0, 1.0, 2.0]])
    return cn,ct

def writeOFDict(fileName,ofDict):
    """ Writes the openfoam dictionary """
    dictOut = ofDictHeader
    dictOut += 'pvj_ref ' + ofDict['ref'] + ';\n'
    dictOut += 'pvj_gamma (' + str(ofDict['gamma']) + ');\n'
    dictOut += 'pvj_bn (\n'
    dictOut += writeCoeff(ofDict['bn']) + '\n'
    dictOut += ');\n\n'
    dictOut += 'pvj_bt1 (\n'
    dictOut += writeCoeff(ofDict['bt1']) + '\n'
    dictOut += ');\n\n'
    
    with open(fileName,'w') as ff:
        ff.write(dictOut)
    
def writeCoeff(bb):
    strOutList = list()
    for iB in range(bb.shape[0]):
        thisLine = '    (' + fmt(bb[iB,0]) + ' ' + fmt(bb[iB,1]) + ' ' + fmt(bb[iB,2]) +')'
        strOutList.append(thisLine)
    return '\n'.join(strOutList)

def getpSurfaceCoefficients(pSurfaceDict):
    """ Function which provides the data needed to fit the barrier behaviour """
    
    pSurfaceData = dict()
    
    if pSurfaceDict['approach'] == 'fullyDeflective':
        bn,bt1 = getFullyDeflective(pSurfaceDict['theta'])
        pSurfaceData['ref'] = 'locRef'
        pSurfaceData['bn'] = bn
        pSurfaceData['bt1'] = bt1
        pSurfaceData['gamma'] = 1.0
        
    elif pSurfaceDict['approach'] == 'fromData':
        pSurfaceData['ref'] = pSurfaceDict['ref']
        pSurfaceData['bn'] = getFourierCoeffs(pSurfaceDict['order'],pSurfaceDict['data'][:,[0,1]])
        pSurfaceData['bt1'] = getFourierCoeffs(pSurfaceDict['order'],pSurfaceDict['data'][:,[0,2]])
        pSurfaceData['gamma'] = 0.0
        
    elif pSurfaceDict['approach'] == 'fromPorosity':
        pSurfaceData['ref'] = pSurfaceDict['ref']
        beta = pSurfaceDict['beta']
        k = (0.5*beta+2.0)*(1.0-beta)/(1.1*beta**2)
        pSurfaceData['bn'] = numpy.array([[0.0, 1.0, k],])
        pSurfaceData['bt1'] = numpy.array([[0.0, 1.0, 0.0],])
        pSurfaceData['gamma'] = 1.0
       
    return pSurfaceData 

        
    
    
    